/*=============================================================================
    Phoenix V1.0
    Copyright (c) 2001-2002 Joel de Guzman

    Permission to copy, use, modify, sell and distribute this software
    is granted provided this copyright notice appears in all copies.
    This software is provided "as is" without express or implied
    warranty, and with no claim as to its suitability for any purpose.
==============================================================================*/
#ifndef PHOENIX_TUPLEHELPERS_HPP
#define PHOENIX_TUPLEHELPERS_HPP

///////////////////////////////////////////////////////////////////////////////
#include <cassert>
#include "boost/phoenix/tuples.hpp"

///////////////////////////////////////////////////////////////////////////////
namespace phoenix
{

///////////////////////////////////////////////////////////////////////////////
//
//  make_tuple template class
//
//      This template class is used to calculate a tuple type required to hold
//      the given template parameter type
//
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
//  normal (non-tuple types are wrapped into a tuple)
template <typename ResultT>
struct make_tuple {

    typedef tuple<ResultT> type;
};

///////////////////////////////////////////////////////////////////////////////
//  nil_t is converted to an empty tuple type
template <>
struct make_tuple<nil_t> {

    typedef tuple<> type;
};

///////////////////////////////////////////////////////////////////////////////
//  tuple types are left alone without any refactoring
template <
      typename A, typename B, typename C
#if PHOENIX_LIMIT > 3
    , typename D, typename E, typename F
#if PHOENIX_LIMIT > 6
    , typename G, typename H, typename I 
#if PHOENIX_LIMIT > 9
    , typename J, typename K, typename L 
#if PHOENIX_LIMIT > 12
    , typename M, typename N, typename O
#endif
#endif
#endif
#endif
>
struct make_tuple<tuple<A, B, C
#if PHOENIX_LIMIT > 3
    , D, E, F
#if PHOENIX_LIMIT > 6
    , G, H, I
#if PHOENIX_LIMIT > 9
    , J, K, L
#if PHOENIX_LIMIT > 12
    , M, N, O
#endif
#endif
#endif
#endif
    > > {

// the tuple parameter itself is the required tuple type
    typedef tuple<A, B, C
#if PHOENIX_LIMIT > 3
        , D, E, F
#if PHOENIX_LIMIT > 6
        , G, H, I
#if PHOENIX_LIMIT > 9
        , J, K, L
#if PHOENIX_LIMIT > 12
        , M, N, O
#endif
#endif
#endif
#endif
        > type;
};

///////////////////////////////////////////////////////////////////////////////
//
//  concat_tuple type computer 
//
//      This class returns the type of a tuple, which is constructed by 
//      concatenating a tuple with a given type
//
///////////////////////////////////////////////////////////////////////////////
template <typename TupleT, typename AppendT>
struct concat_tuple;

///////////////////////////////////////////////////////////////////////////////
//
//  concat tuple <0 member> class
//
///////////////////////////////////////////////////////////////////////////////
template <typename AppendT>
struct concat_tuple<tuple<>, AppendT> {

    typedef tuple<AppendT> type;
};

template <>
struct concat_tuple<tuple<>, nil_t> {

    typedef tuple<> type;
};

///////////////////////////////////////////////////////////////////////////////
//
//  concat tuple <1 member> class
//
///////////////////////////////////////////////////////////////////////////////
template <typename A, typename AppendT>
struct concat_tuple<tuple<A>, AppendT> {

    typedef tuple<A, AppendT> type;
};

template <typename A>
struct concat_tuple<tuple<A>, nil_t> {

    typedef tuple<A> type;
};

///////////////////////////////////////////////////////////////////////////////
//
//  concat tuple <2 member> class
//
///////////////////////////////////////////////////////////////////////////////
template <typename A, typename B, typename AppendT>
struct concat_tuple<tuple<A, B>, AppendT> {

    typedef tuple<A, B, AppendT> type;
};

template <typename A, typename B>
struct concat_tuple<tuple<A, B>, nil_t> {

    typedef tuple<A, B> type;
};

#if PHOENIX_LIMIT > 3
///////////////////////////////////////////////////////////////////////////////
//
//  concat tuple <3 member> class
//
///////////////////////////////////////////////////////////////////////////////
template <
    typename A, typename B, typename C, 
    typename AppendT
>
struct concat_tuple<tuple<A, B, C>, AppendT> {

    typedef tuple<A, B, C, AppendT> type;
};

template <
    typename A, typename B, typename C
>
struct concat_tuple<tuple<A, B, C>, nil_t> {

    typedef tuple<A, B, C> type;
};

///////////////////////////////////////////////////////////////////////////////
//
//  concat tuple <4 member> class
//
///////////////////////////////////////////////////////////////////////////////
template <
    typename A, typename B, typename C, typename D,
    typename AppendT
>
struct concat_tuple<tuple<A, B, C, D>, AppendT> {

    typedef tuple<A, B, C, D, AppendT> type;
};

template <
    typename A, typename B, typename C, typename D
>
struct concat_tuple<tuple<A, B, C, D>, nil_t> {

    typedef tuple<A, B, C, D> type;
};

///////////////////////////////////////////////////////////////////////////////
//
//  concat tuple <5 member> class
//
///////////////////////////////////////////////////////////////////////////////
template <
    typename A, typename B, typename C, typename D, typename E,
    typename AppendT
>
struct concat_tuple<tuple<A, B, C, D, E>, AppendT> {

    typedef tuple<A, B, C, D, E, AppendT> type;
};

template <
    typename A, typename B, typename C, typename D, typename E
>
struct concat_tuple<tuple<A, B, C, D, E>, nil_t> {

    typedef tuple<A, B, C, D, E> type;
};

#if PHOENIX_LIMIT > 6
///////////////////////////////////////////////////////////////////////////////
//
//  concat tuple <6 member> class
//
///////////////////////////////////////////////////////////////////////////////
template <
    typename A, typename B, typename C, typename D, typename E, typename F,
    typename AppendT
>
struct concat_tuple<tuple<A, B, C, D, E, F>, AppendT> {

    typedef tuple<A, B, C, D, E, F, AppendT> type;
};

template <
    typename A, typename B, typename C, typename D, typename E, typename F
>
struct concat_tuple<tuple<A, B, C, D, E, F>, nil_t> {

    typedef tuple<A, B, C, D, E, F> type;
};

///////////////////////////////////////////////////////////////////////////////
//
//  concat tuple <7 member> class
//
///////////////////////////////////////////////////////////////////////////////
template <
    typename A, typename B, typename C, typename D, typename E, typename F,
    typename G,
    typename AppendT
>
struct concat_tuple<tuple<A, B, C, D, E, F, G>, AppendT> {

    typedef tuple<A, B, C, D, E, F, G, AppendT> type;
};

template <
    typename A, typename B, typename C, typename D, typename E, typename F,
    typename G
>
struct concat_tuple<tuple<A, B, C, D, E, F, G>, nil_t> {

    typedef tuple<A, B, C, D, E, F, G> type;
};

///////////////////////////////////////////////////////////////////////////////
//
//  concat tuple <8 member> class
//
///////////////////////////////////////////////////////////////////////////////
template <
    typename A, typename B, typename C, typename D, typename E, typename F,
    typename G, typename H,
    typename AppendT
>
struct concat_tuple<tuple<A, B, C, D, E, F, G, H>, AppendT> {

    typedef tuple<A, B, C, D, E, F, G, H, AppendT> type;
};

template <
    typename A, typename B, typename C, typename D, typename E, typename F,
    typename G, typename H
>
struct concat_tuple<tuple<A, B, C, D, E, F, G, H>, nil_t> {

    typedef tuple<A, B, C, D, E, F, G, H> type;
};

#if PHOENIX_LIMIT > 9
///////////////////////////////////////////////////////////////////////////////
//
//  concat tuple <9 member> class
//
///////////////////////////////////////////////////////////////////////////////
template <
    typename A, typename B, typename C, typename D, typename E, typename F,
    typename G, typename H, typename I,
    typename AppendT
>
struct concat_tuple<tuple<A, B, C, D, E, F, G, H, I>, AppendT> {

    typedef tuple<A, B, C, D, E, F, G, H, I, AppendT> type;
};

template <
    typename A, typename B, typename C, typename D, typename E, typename F,
    typename G, typename H, typename I
>
struct concat_tuple<tuple<A, B, C, D, E, F, G, H, I>, nil_t> {

    typedef tuple<A, B, C, D, E, F, G, H, I> type;
};

///////////////////////////////////////////////////////////////////////////////
//
//  concat tuple <10 member> class
//
///////////////////////////////////////////////////////////////////////////////
template <
    typename A, typename B, typename C, typename D, typename E, typename F,
    typename G, typename H, typename I, typename J,
    typename AppendT
>
struct concat_tuple<tuple<A, B, C, D, E, F, G, H, I, J>, AppendT> {

    typedef tuple<A, B, C, D, E, F, G, H, I, J, AppendT> type;
};

template <
    typename A, typename B, typename C, typename D, typename E, typename F,
    typename G, typename H, typename I, typename J
>
struct concat_tuple<tuple<A, B, C, D, E, F, G, H, I, J>, nil_t> {

    typedef tuple<A, B, C, D, E, F, G, H, I, J> type;
};

///////////////////////////////////////////////////////////////////////////////
//
//  concat tuple <11 member> class
//
///////////////////////////////////////////////////////////////////////////////
template <
    typename A, typename B, typename C, typename D, typename E, typename F,
    typename G, typename H, typename I, typename J, typename K,
    typename AppendT
>
struct concat_tuple<tuple<A, B, C, D, E, F, G, H, I, J, K>, AppendT> {

    typedef tuple<A, B, C, D, E, F, G, H, I, J, K, AppendT> type;
};

template <
    typename A, typename B, typename C, typename D, typename E, typename F,
    typename G, typename H, typename I, typename J, typename K
>
struct concat_tuple<tuple<A, B, C, D, E, F, G, H, I, J, K>, nil_t> {

    typedef tuple<A, B, C, D, E, F, G, H, I, J, K> type;
};

#if PHOENIX_LIMIT > 12
///////////////////////////////////////////////////////////////////////////////
//
//  concat tuple <12 member> class
//
///////////////////////////////////////////////////////////////////////////////
template <
    typename A, typename B, typename C, typename D, typename E, typename F,
    typename G, typename H, typename I, typename J, typename K, typename L,
    typename AppendT
>
struct concat_tuple<tuple<A, B, C, D, E, F, G, H, I, J, K, L>, AppendT> {

    typedef tuple<A, B, C, D, E, F, G, H, I, J, K, L, AppendT> type;
};

template <
    typename A, typename B, typename C, typename D, typename E, typename F,
    typename G, typename H, typename I, typename J, typename K, typename L
>
struct concat_tuple<tuple<A, B, C, D, E, F, G, H, I, J, K, L>, nil_t> {

    typedef tuple<A, B, C, D, E, F, G, H, I, J, K, L> type;
};

///////////////////////////////////////////////////////////////////////////////
//
//  concat tuple <13 member> class
//
///////////////////////////////////////////////////////////////////////////////
template <
    typename A, typename B, typename C, typename D, typename E, typename F,
    typename G, typename H, typename I, typename J, typename K, typename L,
    typename M,
    typename AppendT
>
struct concat_tuple<tuple<A, B, C, D, E, F, G, H, I, J, K, L, M>, AppendT> {

    typedef tuple<A, B, C, D, E, F, G, H, I, J, K, L, M, AppendT> type;
};

template <
    typename A, typename B, typename C, typename D, typename E, typename F,
    typename G, typename H, typename I, typename J, typename K, typename L,
    typename M
>
struct concat_tuple<tuple<A, B, C, D, E, F, G, H, I, J, K, L, M>, nil_t> {

    typedef tuple<A, B, C, D, E, F, G, H, I, J, K, L, M> type;
};

///////////////////////////////////////////////////////////////////////////////
//
//  concat tuple <14 member> class
//
///////////////////////////////////////////////////////////////////////////////
template <
    typename A, typename B, typename C, typename D, typename E, typename F,
    typename G, typename H, typename I, typename J, typename K, typename L,
    typename M, typename N,
    typename AppendT
>
struct concat_tuple<tuple<A, B, C, D, E, F, G, H, I, J, K, L, M, N>, AppendT> {

    typedef tuple<A, B, C, D, E, F, G, H, I, J, K, L, M, N, AppendT> type;
};

template <
    typename A, typename B, typename C, typename D, typename E, typename F,
    typename G, typename H, typename I, typename J, typename K, typename L,
    typename M, typename N
>
struct concat_tuple<tuple<A, B, C, D, E, F, G, H, I, J, K, L, M, N>, nil_t> {

    typedef tuple<A, B, C, D, E, F, G, H, I, J, K, L, M, N> type;
};

#endif
#endif
#endif
#endif

///////////////////////////////////////////////////////////////////////////////
//
//  concat_tuples type computer 
//
//      This template class returns the type of a tuple build from the 
//      concatenation of to given tuples.
//
///////////////////////////////////////////////////////////////////////////////
template <typename TupleT1, typename TupleT2, int N, typename AppendT>
struct concat_tuple_element {

    typedef 
        typename concat_tuple_element<
                typename concat_tuple<TupleT1, AppendT>::type, TupleT2, N+1, 
                typename tuple_element<N+1, TupleT2>::type
            >::type
        type;
};

template <typename TupleT1, typename TupleT2, int N>
struct concat_tuple_element<TupleT1, TupleT2, N, nil_t> {

    typedef TupleT1 type;
};

template <typename TupleT1, typename TupleT2>
struct concat_tuples {

    typedef 
        typename concat_tuple_element<
                TupleT1, TupleT2, 0, 
                typename tuple_element<0, TupleT2>::type
            >::type
        type;
};

namespace impl {
///////////////////////////////////////////////////////////////////////////////
//
//  assign_tuple_member
//
//      Dynamically access single tuple member by it's number.
//
//  Implementation note:
//      Be aware: this implementation uses bad programming techniques! I see
//      no other way to achieve the required functionality without switching
//      of the C++ type checking system.
//      The type safety should be ensured through external (to this function)
//      methods.
//      The asserts ensure a minimal amount of safety only.
//
///////////////////////////////////////////////////////////////////////////////
    template <int N, typename T>
    struct dyn_tuple_member;
    
    template <typename T>
    struct dyn_tuple_member<1, T> {
    
        template <typename A>
        static void *
        access(tuple<A> &t, int member)
        {
            switch (member) {
            case 0:     assert(sizeof(T) == sizeof(t.a)); return &t.a; 
            default:
                assert(member == 0);
                return 0;
            }
        }
    };

    template <typename T>
    struct dyn_tuple_member<2, T> {
    
        template <typename A, typename B>
        static void *
        access(tuple<A, B> &t, int member)
        {
            switch (member) {
            case 0:     assert(sizeof(T) == sizeof(t.a)); return &t.a; 
            case 1:     assert(sizeof(T) == sizeof(t.b)); return &t.b; 
            default:
                assert(member >= 0 && member <= 1);
                return 0;
            }
        }
    };
    
    template <typename T>
    struct dyn_tuple_member<3, T> {
    
        template <typename A, typename B, typename C>
        static void *
        access(tuple<A, B, C> &t, int member)
        {
            switch (member) {
            case 0:     assert(sizeof(T) == sizeof(t.a)); return &t.a; 
            case 1:     assert(sizeof(T) == sizeof(t.b)); return &t.b; 
            case 2:     assert(sizeof(T) == sizeof(t.c)); return &t.c; 
            default:
                assert(member >= 0 && member <= 2);
                return 0;
            }
        }
    };
    
#if PHOENIX_LIMIT > 3
    template <typename T>
    struct dyn_tuple_member<4, T> {
    
        template <typename A, typename B, typename C, typename D>
        static void *
        access(tuple<A, B, C, D> &t, int member)
        {
            switch (member) {
            case 0:     assert(sizeof(T) == sizeof(t.a)); return &t.a; 
            case 1:     assert(sizeof(T) == sizeof(t.b)); return &t.b; 
            case 2:     assert(sizeof(T) == sizeof(t.c)); return &t.c; 
            case 3:     assert(sizeof(T) == sizeof(t.d)); return &t.d; 
            default:
                assert(member >= 0 && member <= 3);
                return 0;
            }
        }
    };
    
    template <typename T>
    struct dyn_tuple_member<5, T> {
    
        template <
            typename A, typename B, typename C, typename D, typename E
        >
        static void *
        access(tuple<A, B, C, D, E> &t, int member)
        {
            switch (member) {
            case 0:     assert(sizeof(T) == sizeof(t.a)); return &t.a; 
            case 1:     assert(sizeof(T) == sizeof(t.b)); return &t.b; 
            case 2:     assert(sizeof(T) == sizeof(t.c)); return &t.c; 
            case 3:     assert(sizeof(T) == sizeof(t.d)); return &t.d; 
            case 4:     assert(sizeof(T) == sizeof(t.e)); return &t.e; 
            default:
                assert(member >= 0 && member <= 4);
                return 0;
            }
        }
    };
    
    template <typename T>
    struct dyn_tuple_member<6, T> {
    
        template <
            typename A, typename B, typename C, typename D, typename E,
            typename F
        >
        static void *
        access(tuple<A, B, C, D, E, F> &t, int member)
        {
            switch (member) {
            case 0:     assert(sizeof(T) == sizeof(t.a)); return &t.a; 
            case 1:     assert(sizeof(T) == sizeof(t.b)); return &t.b; 
            case 2:     assert(sizeof(T) == sizeof(t.c)); return &t.c; 
            case 3:     assert(sizeof(T) == sizeof(t.d)); return &t.d; 
            case 4:     assert(sizeof(T) == sizeof(t.e)); return &t.e; 
            case 5:     assert(sizeof(T) == sizeof(t.f)); return &t.f; 
            default:
                assert(member >= 0 && member <= 5);
                return 0;
            }
        }
    };
    
#if PHOENIX_LIMIT > 6
    template <typename T>
    struct dyn_tuple_member<7, T> {
    
        template <
            typename A, typename B, typename C, typename D, typename E,
            typename F, typename G
        >
        static void *
        access(tuple<A, B, C, D, E, F, G> &t, int member)
        {
            switch (member) {
            case 0:     assert(sizeof(T) == sizeof(t.a)); return &t.a; 
            case 1:     assert(sizeof(T) == sizeof(t.b)); return &t.b; 
            case 2:     assert(sizeof(T) == sizeof(t.c)); return &t.c; 
            case 3:     assert(sizeof(T) == sizeof(t.d)); return &t.d; 
            case 4:     assert(sizeof(T) == sizeof(t.e)); return &t.e; 
            case 5:     assert(sizeof(T) == sizeof(t.f)); return &t.f; 
            case 6:     assert(sizeof(T) == sizeof(t.g)); return &t.g; 
            default:
                assert(member >= 0 && member <= 6);
                return 0;
            }
        }
    };
    
    template <typename T>
    struct dyn_tuple_member<8, T> {
    
        template <
            typename A, typename B, typename C, typename D, typename E,
            typename F, typename G, typename H
        >
        static void *
        access(tuple<A, B, C, D, E, F, G, H> &t, int member)
        {
            switch (member) {
            case 0:     assert(sizeof(T) == sizeof(t.a)); return &t.a; 
            case 1:     assert(sizeof(T) == sizeof(t.b)); return &t.b; 
            case 2:     assert(sizeof(T) == sizeof(t.c)); return &t.c; 
            case 3:     assert(sizeof(T) == sizeof(t.d)); return &t.d; 
            case 4:     assert(sizeof(T) == sizeof(t.e)); return &t.e; 
            case 5:     assert(sizeof(T) == sizeof(t.f)); return &t.f; 
            case 6:     assert(sizeof(T) == sizeof(t.g)); return &t.g; 
            case 7:     assert(sizeof(T) == sizeof(t.h)); return &t.h; 
            default:
                assert(member >= 0 && member <= 7);
                return 0;
            }
        }
    };
    
    template <typename T>
    struct dyn_tuple_member<9, T> {
    
        template <
            typename A, typename B, typename C, typename D, typename E,
            typename F, typename G, typename H, typename I
        >
        static void *
        access(tuple<A, B, C, D, E, F, G, H, I> &t, int member)
        {
            switch (member) {
            case 0:     assert(sizeof(T) == sizeof(t.a)); return &t.a; 
            case 1:     assert(sizeof(T) == sizeof(t.b)); return &t.b; 
            case 2:     assert(sizeof(T) == sizeof(t.c)); return &t.c; 
            case 3:     assert(sizeof(T) == sizeof(t.d)); return &t.d; 
            case 4:     assert(sizeof(T) == sizeof(t.e)); return &t.e; 
            case 5:     assert(sizeof(T) == sizeof(t.f)); return &t.f; 
            case 6:     assert(sizeof(T) == sizeof(t.g)); return &t.g; 
            case 7:     assert(sizeof(T) == sizeof(t.h)); return &t.h; 
            case 8:     assert(sizeof(T) == sizeof(t.i)); return &t.i; 
            default:
                assert(member >= 0 && member <= 8);
                return 0;
            }
        }
    };
    
#if PHOENIX_LIMIT > 9
    template <typename T>
    struct dyn_tuple_member<10, T> {
    
        template <
            typename A, typename B, typename C, typename D, typename E,
            typename F, typename G, typename H, typename I, typename J
        >
        static void *
        access(tuple<A, B, C, D, E, F, G, H, I, J> &t, int member)
        {
            switch (member) {
            case 0:     assert(sizeof(T) == sizeof(t.a)); return &t.a; 
            case 1:     assert(sizeof(T) == sizeof(t.b)); return &t.b; 
            case 2:     assert(sizeof(T) == sizeof(t.c)); return &t.c; 
            case 3:     assert(sizeof(T) == sizeof(t.d)); return &t.d; 
            case 4:     assert(sizeof(T) == sizeof(t.e)); return &t.e; 
            case 5:     assert(sizeof(T) == sizeof(t.f)); return &t.f; 
            case 6:     assert(sizeof(T) == sizeof(t.g)); return &t.g; 
            case 7:     assert(sizeof(T) == sizeof(t.h)); return &t.h; 
            case 8:     assert(sizeof(T) == sizeof(t.i)); return &t.i; 
            case 9:     assert(sizeof(T) == sizeof(t.j)); return &t.j; 
            default:
                assert(member >= 0 && member <= 9);
                return 0;
            }
        }
    };
    
    template <typename T>
    struct dyn_tuple_member<11, T> {
    
        template <
            typename A, typename B, typename C, typename D, typename E,
            typename F, typename G, typename H, typename I, typename J,
            typename K
        >
        static void *
        access(tuple<A, B, C, D, E, F, G, H, I, J, K> &t, int member)
        {
            switch (member) {
            case 0:     assert(sizeof(T) == sizeof(t.a)); return &t.a; 
            case 1:     assert(sizeof(T) == sizeof(t.b)); return &t.b; 
            case 2:     assert(sizeof(T) == sizeof(t.c)); return &t.c; 
            case 3:     assert(sizeof(T) == sizeof(t.d)); return &t.d; 
            case 4:     assert(sizeof(T) == sizeof(t.e)); return &t.e; 
            case 5:     assert(sizeof(T) == sizeof(t.f)); return &t.f; 
            case 6:     assert(sizeof(T) == sizeof(t.g)); return &t.g; 
            case 7:     assert(sizeof(T) == sizeof(t.h)); return &t.h; 
            case 8:     assert(sizeof(T) == sizeof(t.i)); return &t.i; 
            case 9:     assert(sizeof(T) == sizeof(t.j)); return &t.j; 
            case 10:    assert(sizeof(T) == sizeof(t.k)); return &t.k; 
            default:
                assert(member >= 0 && member <= 10);
                return 0;
            }
        }
    };
    
    template <typename T>
    struct dyn_tuple_member<12, T> {
    
        template <
            typename A, typename B, typename C, typename D, typename E,
            typename F, typename G, typename H, typename I, typename J,
            typename K, typename L
        >
        static void *
        access(tuple<A, B, C, D, E, F, G, H, I, J, K, L> &t, int member)
        {
            switch (member) {
            case 0:     assert(sizeof(T) == sizeof(t.a)); return &t.a; 
            case 1:     assert(sizeof(T) == sizeof(t.b)); return &t.b; 
            case 2:     assert(sizeof(T) == sizeof(t.c)); return &t.c; 
            case 3:     assert(sizeof(T) == sizeof(t.d)); return &t.d; 
            case 4:     assert(sizeof(T) == sizeof(t.e)); return &t.e; 
            case 5:     assert(sizeof(T) == sizeof(t.f)); return &t.f; 
            case 6:     assert(sizeof(T) == sizeof(t.g)); return &t.g; 
            case 7:     assert(sizeof(T) == sizeof(t.h)); return &t.h; 
            case 8:     assert(sizeof(T) == sizeof(t.i)); return &t.i; 
            case 9:     assert(sizeof(T) == sizeof(t.j)); return &t.j; 
            case 10:    assert(sizeof(T) == sizeof(t.k)); return &t.k; 
            case 11:    assert(sizeof(T) == sizeof(t.l)); return &t.l; 
            default:
                assert(member >= 0 && member <= 11);
                return 0;
            }
        }
    };
    
#if PHOENIX_LIMIT > 12
    template <typename T>
    struct dyn_tuple_member<13, T> {

        template <
            typename A, typename B, typename C, typename D, typename E,
            typename F, typename G, typename H, typename I, typename J,
            typename K, typename L, typename M
        >
        static void *
        access(tuple<A, B, C, D, E, F, G, H, I, J, K, L, M> &t, int member)
        {
            switch (member) {
            case 0:     assert(sizeof(T) == sizeof(t.a)); return &t.a; 
            case 1:     assert(sizeof(T) == sizeof(t.b)); return &t.b; 
            case 2:     assert(sizeof(T) == sizeof(t.c)); return &t.c; 
            case 3:     assert(sizeof(T) == sizeof(t.d)); return &t.d; 
            case 4:     assert(sizeof(T) == sizeof(t.e)); return &t.e; 
            case 5:     assert(sizeof(T) == sizeof(t.f)); return &t.f; 
            case 6:     assert(sizeof(T) == sizeof(t.g)); return &t.g; 
            case 7:     assert(sizeof(T) == sizeof(t.h)); return &t.h; 
            case 8:     assert(sizeof(T) == sizeof(t.i)); return &t.i; 
            case 9:     assert(sizeof(T) == sizeof(t.j)); return &t.j; 
            case 10:    assert(sizeof(T) == sizeof(t.k)); return &t.k; 
            case 11:    assert(sizeof(T) == sizeof(t.l)); return &t.l; 
            case 12:    assert(sizeof(T) == sizeof(t.m)); return &t.m; 
            default:
                assert(member >= 0 && member <= 12);
                return 0;
            }
        }
    };
    
    template <typename T>
    struct dyn_tuple_member<14, T> {

        template <
            typename A, typename B, typename C, typename D, typename E,
            typename F, typename G, typename H, typename I, typename J,
            typename K, typename L, typename M, typename N
        >
        static void *
        access(tuple<A, B, C, D, E, F, G, H, I, J, K, L, M, N> &t, int member)
        {
            switch (member) {
            case 0:     assert(sizeof(T) == sizeof(t.a)); return &t.a; 
            case 1:     assert(sizeof(T) == sizeof(t.b)); return &t.b; 
            case 2:     assert(sizeof(T) == sizeof(t.c)); return &t.c; 
            case 3:     assert(sizeof(T) == sizeof(t.d)); return &t.d; 
            case 4:     assert(sizeof(T) == sizeof(t.e)); return &t.e; 
            case 5:     assert(sizeof(T) == sizeof(t.f)); return &t.f; 
            case 6:     assert(sizeof(T) == sizeof(t.g)); return &t.g; 
            case 7:     assert(sizeof(T) == sizeof(t.h)); return &t.h; 
            case 8:     assert(sizeof(T) == sizeof(t.i)); return &t.i; 
            case 9:     assert(sizeof(T) == sizeof(t.j)); return &t.j; 
            case 10:    assert(sizeof(T) == sizeof(t.k)); return &t.k; 
            case 11:    assert(sizeof(T) == sizeof(t.l)); return &t.l; 
            case 12:    assert(sizeof(T) == sizeof(t.m)); return &t.m; 
            case 13:    assert(sizeof(T) == sizeof(t.n)); return &t.n; 
            default:
                assert(member >= 0 && member <= 13);
                return 0;
            }
        }
    };
    
    template <typename T>
    struct dyn_tuple_member<15, T> {

        template <
            typename A, typename B, typename C, typename D, typename E,
            typename F, typename G, typename H, typename I, typename J,
            typename K, typename L, typename M, typename N, typename O
        >
        static void *
        access(tuple<A, B, C, D, E, F, G, H, I, J, K, L, M, N, O> &t, int member)
        {
            switch (member) {
            case 0:     assert(sizeof(T) == sizeof(t.a)); return &t.a; 
            case 1:     assert(sizeof(T) == sizeof(t.b)); return &t.b; 
            case 2:     assert(sizeof(T) == sizeof(t.c)); return &t.c; 
            case 3:     assert(sizeof(T) == sizeof(t.d)); return &t.d; 
            case 4:     assert(sizeof(T) == sizeof(t.e)); return &t.e; 
            case 5:     assert(sizeof(T) == sizeof(t.f)); return &t.f; 
            case 6:     assert(sizeof(T) == sizeof(t.g)); return &t.g; 
            case 7:     assert(sizeof(T) == sizeof(t.h)); return &t.h; 
            case 8:     assert(sizeof(T) == sizeof(t.i)); return &t.i; 
            case 9:     assert(sizeof(T) == sizeof(t.j)); return &t.j; 
            case 10:    assert(sizeof(T) == sizeof(t.k)); return &t.k; 
            case 11:    assert(sizeof(T) == sizeof(t.l)); return &t.l; 
            case 12:    assert(sizeof(T) == sizeof(t.m)); return &t.m; 
            case 13:    assert(sizeof(T) == sizeof(t.n)); return &t.n; 
            case 14:    assert(sizeof(T) == sizeof(t.o)); return &t.o; 
            default:
                assert(member >= 0 && member <= 14);
                return 0;
            }
        }
    };
    
#endif
#endif
#endif
#endif

///////////////////////////////////////////////////////////////////////////////
}   // namespace impl

///////////////////////////////////////////////////////////////////////////////
}   //  namespace phoenix

#endif // PHOENIX_TUPLEHELPERS_HPP
