/*=============================================================================
    Spirit V1.5.0
    Copyright (c) 1998-2002 Spirit-Dev
    http://spirit.sourceforge.net/

    Permission to copy, use, modify, sell and distribute this software
    is granted provided this copyright notice appears in all copies.
    This software is provided "as is" without express or implied
    warranty, and with no claim as to its suitability for any purpose.

    See spirit.hpp for full copyright notices.
=============================================================================*/
#ifndef SPIRIT_GROUPING_HPP
#define SPIRIT_GROUPING_HPP

#include "boost/spirit/core/parser.hpp"
#include "boost/phoenix/tuples.hpp"
#include "boost/phoenix/tuple_helpers.hpp"

#include "boost/spirit/attribute/impl/grouping.ipp"

///////////////////////////////////////////////////////////////////////////////
namespace spirit
{

///////////////////////////////////////////////////////////////////////////////
//
//  grouping_result class
//
///////////////////////////////////////////////////////////////////////////////
template <typename ParserT, typename ScannerT, typename CategoryT>
struct grouping_result {

    typedef typename impl::tuple_result<CategoryT>
        ::template result<ParserT, ScannerT>::type type;
};

///////////////////////////////////////////////////////////////////////////////
//
//  grouping_parser class
//
///////////////////////////////////////////////////////////////////////////////
struct grouping_parser_gen;

template <typename ParserT>
struct grouping_parser 
:   public unary<ParserT, parser<grouping_parser<ParserT> > >
{
    typedef grouping_parser<ParserT>                    self_t;
    typedef grouping_parser_gen                         parser_generator_t;
    typedef unary_parser_category                       parser_category_t;
    
    typedef unary<ParserT, parser<self_t> >             base_t;
    typedef typename ParserT::parser_category_t         base_category_t;

    template <typename ScannerT>
    struct result
    {
        typedef 
            typename grouping_result<ParserT, ScannerT, base_category_t>::type
            tuple_t;
        typedef typename match_result<ScannerT, tuple_t>::type type;
    };

    grouping_parser()
    : base_t(ParserT()) {}
    
    grouping_parser(ParserT const &p)
    : base_t(p) {}
    
    template <typename ScannerT>
    typename parser_result<self_t, ScannerT>::type
    parse(ScannerT const& scan) const
    {
        impl::grouped_parser<ParserT, ScannerT, self_t, base_category_t> p;
        return p.parse(this->subject(), scan);
    }
};

struct grouping_parser_gen {

    template <typename ParserT>
    static grouping_parser<ParserT>
    generate (parser<ParserT> const &p)
    {
        return grouping_parser<ParserT>(p.derived());
    }

    template <typename ParserT>
    grouping_parser<ParserT>
    operator[] (parser<ParserT> const &p) const
    {
        return grouping_parser<ParserT>(p.derived());
    }
};

const grouping_parser_gen group_d = grouping_parser_gen();

///////////////////////////////////////////////////////////////////////////////
}   //  namespace Spirit

#endif // SPIRIT_GROUPING_HPP
