/*=============================================================================
    Spirit V1.5.0
    Copyright (c) 1998-2002 Spirit-Dev
    http://spirit.sourceforge.net/

    Permission to copy, use, modify, sell and distribute this software
    is granted provided this copyright notice appears in all copies.
    This software is provided "as is" without express or implied
    warranty, and with no claim as to its suitability for any purpose.

    See spirit.hpp for full copyright notices.
=============================================================================*/
#ifndef SPIRIT_PARAMETRIC_HPP
#define SPIRIT_PARAMETRIC_HPP

///////////////////////////////////////////////////////////////////////////////
#include "boost/spirit/core/parser.hpp"
#include "boost/spirit/core/composite.hpp"
#include "boost/spirit/core/primitives.hpp"

///////////////////////////////////////////////////////////////////////////////
namespace spirit
{
///////////////////////////////////////////////////////////////////////////////
//
//  fchlit class [ functional version of chlit ]
//
///////////////////////////////////////////////////////////////////////////////
template <typename ChGenT>
struct fchlit : public char_parser<fchlit<ChGenT> >
{
    fchlit(ChGenT chgen_)
    : chgen(chgen_) {}

    template <typename T>
    bool test(T ch) const
    { return ch == chgen(); }

    ChGenT   chgen;
};

//////////////////////////////////
template <typename ChGenT>
inline fchlit<ChGenT>
fch_p(ChGenT chgen)
{ return fchlit<ChGenT>(chgen); }

///////////////////////////////////////////////////////////////////////////////
//
//  frange class [ functional version of range ]
//
///////////////////////////////////////////////////////////////////////////////
template <typename ChGenAT, typename ChGenBT>
struct frange : public char_parser<frange<ChGenAT, ChGenBT> >
{
    frange(ChGenAT first_, ChGenBT last_)
    : first(first_), last(last_)
    {}

    template <typename T>
    bool test(T ch) const
    {
        SPIRIT_ASSERT(first() <= last());
        return (ch >= first()) && (ch <= last());
    }

    ChGenAT first;
    ChGenBT last;
};

//////////////////////////////////
template <typename ChGenAT, typename ChGenBT>
inline frange<ChGenAT, ChGenBT>
frange_p(ChGenAT first, ChGenBT last)
{ return frange<ChGenAT, ChGenBT>(first, last); }

///////////////////////////////////////////////////////////////////////////////
//
//  fchseq class [ functional version of chseq ]
//
///////////////////////////////////////////////////////////////////////////////
template <typename IterGenAT, typename IterGenBT>
class fchseq : public parser<fchseq<IterGenAT, IterGenBT> >
{
public:

    typedef fchseq<IterGenAT, IterGenBT> self_t;

    fchseq(IterGenAT first_, IterGenBT last_)
    : first(first_), last(last_) {}

    template <typename ScannerT>
    typename parser_result<self_t, ScannerT>::type
    parse(ScannerT const& scan) const
    {
        typedef typename parser_result<self_t, ScannerT>::type result_t;
        return impl::string_parser_parse<result_t>(first(), last(), scan);
    }

private:

    IterGenAT first;
    IterGenBT last;
};

//////////////////////////////////
template <typename IterGenAT, typename IterGenBT>
inline fchseq<IterGenAT, IterGenBT>
fchseq_p(IterGenAT first, IterGenBT last)
{ return fchseq<IterGenAT, IterGenBT>(first, last); }

///////////////////////////////////////////////////////////////////////////////
//
//  fstrlit class [ functional version of strlit ]
//
///////////////////////////////////////////////////////////////////////////////
template <typename IterGenAT, typename IterGenBT>
class fstrlit : public parser<fstrlit<IterGenAT, IterGenBT> >
{
public:

    typedef fstrlit<IterGenAT, IterGenBT> self_t;

    fstrlit(IterGenAT first, IterGenBT last)
    : seq(first, last) {}

    template <typename ScannerT>
    typename parser_result<self_t, ScannerT>::type
    parse(ScannerT const& scan) const
    {
        typedef typename parser_result<self_t, ScannerT>::type result_t;
        return impl::contiguous_parser_parse<result_t>
            (seq, scan, scan);
    }

private:

    fchseq<IterGenAT, IterGenBT> seq;
};

//////////////////////////////////
template <typename IterGenAT, typename IterGenBT>
inline fstrlit<IterGenAT, IterGenBT>
fstr_p(IterGenAT first, IterGenBT last)
{ return fstrlit<IterGenAT, IterGenBT>(first, last); }

///////////////////////////////////////////////////////////////////////////////
}   //  namespace Spirit

#endif
