/*=============================================================================
    Spirit V1.5.0
    Copyright (c) 1998-2002 Spirit-Dev
    MT code Copyright (c) 2002 Martin Wille
    http://spirit.sourceforge.net/

    Permission to copy, use, modify, sell and distribute this software
    is granted provided this copyright notice appears in all copies.
    This software is provided "as is" without express or implied
    warranty, and with no claim as to its suitability for any purpose.

    See spirit.hpp for full copyright notices.
=============================================================================*/
#if !defined(SPIRIT_GRAMMAR_HPP)
#define SPIRIT_GRAMMAR_HPP

///////////////////////////////////////////////////////////////////////////////
#if defined(SPIRIT_THREADSAFE) && defined(SPIRIT_SINGLE_GRAMMAR_INSTANCE)
#undef SPIRIT_SINGLE_GRAMMAR_INSTANCE
#endif

#include "boost/spirit/core/parser.hpp"
#include "boost/spirit/core/parser_context.hpp"
#include "boost/spirit/core/impl/grammar.ipp"

///////////////////////////////////////////////////////////////////////////////
namespace spirit
{
///////////////////////////////////////////////////////////////////////////////
//
//  grammar class
//
///////////////////////////////////////////////////////////////////////////////
template <typename DerivedT, typename ContextT = parser_context>
struct grammar : public parser<DerivedT>, public ContextT::base_t
#if !defined(SPIRIT_SINGLE_GRAMMAR_INSTANCE)
, public impl::object_with_id<impl::grammar_tag>
#endif
{
    typedef grammar<DerivedT, ContextT>     self_t;
    typedef DerivedT const&                 embed_t;
    typedef parser_context_linker<ContextT> context_t;
    typedef typename context_t::attr_t      attr_t;

    template <typename ScannerT>
    struct result
    {
        typedef typename match_result<ScannerT, attr_t>::type type;
    };

    grammar() {}
    ~grammar() { impl::grammar_destruct(this); }

    template <typename ScannerT>
    typename parser_result<self_t, ScannerT>::type
    parse(ScannerT const& scan) const
    { return impl::grammar_parser_parse(this, scan); }

#if !defined(SPIRIT_SINGLE_GRAMMAR_INSTANCE)
    friend struct impl::grammar_extract_helper_list;

# if !defined(__GNUC__) || (__GNUC__ > 2)
private:
# endif

    mutable impl::grammar_helper_list<self_t> helpers;
#endif
};

///////////////////////////////////////////////////////////////////////////////
}   //  namespace Spirit

#endif

