/*=============================================================================
    Spirit V1.5.0
    Copyright (c) 1998-2002 Spirit-Dev
    http://spirit.sourceforge.net/

    Permission to copy, use, modify, sell and distribute this software
    is granted provided this copyright notice appears in all copies.
    This software is provided "as is" without express or implied
    warranty, and with no claim as to its suitability for any purpose.

    See spirit.hpp for full copyright notices.
=============================================================================*/
#if !defined(SPIRIT_DIRECTIVES_IPP)
#define SPIRIT_DIRECTIVES_IPP

///////////////////////////////////////////////////////////////////////////////
#include "boost/spirit/core/skipper.hpp"

#if defined(BOOST_MSVC) && (BOOST_MSVC <= 1300)
#define SPIRIT_TYPENAME
#else
#define SPIRIT_TYPENAME typename
#endif

///////////////////////////////////////////////////////////////////////////////
namespace spirit {

template <typename BaseT>
struct no_skipper_iteration_policy;

template <typename BaseT>
struct inhibit_case_iteration_policy;

template <typename A, typename B>
struct alternative;

template <typename A, typename B>
struct longest_alternative;

template <typename A, typename B>
struct shortest_alternative;

    namespace impl
    {
        template <typename RT, typename ST, typename ScannerT, typename BaseT>
        inline RT
        contiguous_parser_parse(
            ST const& s,
            ScannerT const& scan,
            skipper_iteration_policy<BaseT> const&)
        {
            typedef scanner_policies<
                no_skipper_iteration_policy<
                    SPIRIT_TYPENAME ScannerT::iteration_policy_t>,
                SPIRIT_TYPENAME ScannerT::match_policy_t,
                SPIRIT_TYPENAME ScannerT::action_policy_t
            > policies_t;

            scan.skip(scan);
            RT hit = s.parse(scan.change_policies(policies_t(scan)));
            scan.skip(scan);
            return hit;
        }

        template <typename RT, typename ST, typename ScannerT, typename BaseT>
        inline RT
        contiguous_parser_parse(
            ST const& s,
            ScannerT const& scan,
            no_skipper_iteration_policy<BaseT> const&)
        {
            return s.parse(scan);
        }

        template <typename RT, typename ST, typename ScannerT>
        inline RT
        contiguous_parser_parse(
            ST const& s,
            ScannerT const& scan,
            iteration_policy const&)
        {
            return s.parse(scan);
        }

        template <typename RT, typename ParserT, typename ScannerT, typename BaseT>
        inline RT
        implicit_lexeme_parse(
            ParserT const& p,
            ScannerT const& scan,
            skipper_iteration_policy<BaseT> const&)
        {
            typedef scanner_policies<
                no_skipper_iteration_policy<
                    SPIRIT_TYPENAME ScannerT::iteration_policy_t>,
                SPIRIT_TYPENAME ScannerT::match_policy_t,
                SPIRIT_TYPENAME ScannerT::action_policy_t
            > policies_t;

            scan.skip(scan);
            RT hit = p.parse_main(scan.change_policies(policies_t(scan)));
            scan.skip(scan);
            return hit;
        }

        template <typename RT, typename ParserT, typename ScannerT, typename BaseT>
        inline RT
        implicit_lexeme_parse(
            ParserT const& p,
            ScannerT const& scan,
            no_skipper_iteration_policy<BaseT> const&)
        {
            return p.parse_main(scan);
        }

        template <typename RT, typename ParserT, typename ScannerT>
        inline RT
        implicit_lexeme_parse(
            ParserT const& p,
            ScannerT const& scan,
            iteration_policy const&)
        {
            return p.parse_main(scan);
        }

        template <typename RT, typename ST, typename ScannerT>
        inline RT
        inhibit_case_parser_parse(
            ST const& s,
            ScannerT const& scan,
            iteration_policy const&)
        {
            typedef scanner_policies<
                inhibit_case_iteration_policy<
                    SPIRIT_TYPENAME ScannerT::iteration_policy_t>,
                SPIRIT_TYPENAME ScannerT::match_policy_t,
                SPIRIT_TYPENAME ScannerT::action_policy_t
            > policies_t;

            return s.parse(scan.change_policies(policies_t(scan)));
        }

        template <typename RT, typename ST, typename ScannerT, typename BaseT>
        inline RT
        inhibit_case_parser_parse(
            ST const& s,
            ScannerT const& scan,
            inhibit_case_iteration_policy<BaseT> const&)
        {
            return s.parse(scan);
        }

#if defined(BOOST_NO_TEMPLATE_PARTIAL_SPECIALIZATION)

        ///////////////////////////////////////////////////////////////////////
        //
        //  from spirit 1.1 (copyright (c) 2001 Bruce Florman)
        //  various workarounds to support longest and shortest directives
        //
        ///////////////////////////////////////////////////////////////////////
        template <typename T>
        struct is_alternative
        {
        //  Determine at compile time (without partial specialization)
        //  whether a given type is an instance of the alternative<A,B>

            static T t();
            template <typename A, typename B>
            static char test_(alternative<A, B> const&);    // no implementation
            static int  test_(...);                         // no implementation
            enum { r = sizeof(char) == sizeof(test_(t())) };
            typedef bool_t<r> value;
        };

        template <typename T> struct select_to_longest;

        template <typename T>
        struct to_longest_alternative
        {
            typedef typename select_to_longest<T>::result_t result_t;
            typedef typename select_to_longest<T>::plain_t  plain_t;
            typedef typename select_to_longest<T>::choose_t choose_t;
            static result_t convert(T const& a);
        };

        template <typename T>
        struct to_longest_generic
        {
            typedef T const&    result_t;
            typedef T           plain_t;
            typedef false_t     choose_t;
        };

        template <typename T>
        inline T const&
        to_longest_convert(T const& a, false_t)
        { return a; }

        template <typename T>
        struct to_longest_recursive
        {
            typedef typename to_longest_alternative<
                typename T::left_t>::plain_t    a_t;
            typedef typename to_longest_alternative<
                typename T::right_t>::plain_t   b_t;

            typedef longest_alternative<a_t, b_t>   result_t;

            typedef result_t plain_t;
            typedef true_t choose_t;
        };

        template <typename A, typename B>
        inline typename to_longest_alternative<alternative<A, B> >::result_t
        to_longest_convert(alternative<A, B> const& alt, true_t)
        {
            typedef typename to_longest_alternative<
                alternative<A, B> >::result_t result_t;
            return result_t(
                to_longest_alternative<A>::convert(alt.left()),
                to_longest_alternative<B>::convert(alt.right()));
        }

        template <typename T>
        inline typename to_longest_alternative<T>::result_t
        to_longest_alternative<T>::convert(T const& a)
        {
            return to_longest_convert(
                a, to_longest_alternative<T>::choose_t());
        }

        template <typename T>
        struct select_to_longest
        {
            typedef typename if_t
            <
                is_alternative<T>::value    //  IF
                , to_longest_recursive<T>   //  THEN
                , to_longest_generic<T>     //  ELSE
            >::type type;

            typedef typename type::result_t result_t;
            typedef typename type::plain_t  plain_t;
            typedef typename type::choose_t choose_t;
        };

        template <typename T> struct select_to_shortest;

        template <typename T>
        struct to_shortest_alternative
        {
            typedef typename select_to_shortest<T>::result_t    result_t;
            typedef typename select_to_shortest<T>::plain_t     plain_t;
            typedef typename select_to_shortest<T>::choose_t    choose_t;
            static result_t convert(T const& a);
        };

        template <typename T>
        struct to_shortest_generic
        {
            typedef T const&    result_t;
            typedef T           plain_t;
            typedef false_t     choose_t;
        };

        template <typename T>
        inline T const&
        to_shortest_convert(T const& a, false_t) { return a; }

        template <typename T>
        struct to_shortest_recursive
        {
            typedef typename to_shortest_alternative<
                typename T::left_t>::plain_t    a_t;
            typedef typename to_shortest_alternative<
                typename T::right_t>::plain_t   b_t;

            typedef shortest_alternative<a_t, b_t>  result_t;

            typedef result_t    plain_t;
            typedef true_t      choose_t;
        };

        template <typename A, typename B>
        inline typename to_shortest_alternative<alternative<A, B> >::result_t
        to_shortest_convert(alternative<A, B> const& alt, true_t)
        {
            typedef typename to_shortest_alternative<
                alternative<A, B> >::result_t result_t;
            return result_t(
                to_shortest_alternative<A>::convert(alt.left()),
                to_shortest_alternative<B>::convert(alt.right()));
        }

        template <typename T>
        inline typename to_shortest_alternative<T>::result_t
        to_shortest_alternative<T>::convert(T const& a)
        {
            return to_shortest_convert(
                a, to_shortest_alternative<T>::choose_t());
        }

        template <typename T>
        struct select_to_shortest
        {
            typedef typename if_t
            <
                is_alternative<T>::value    //  IF
                , to_shortest_recursive<T>  //  THEN
                , to_shortest_generic<T>    //  ELSE
            >::type type;

            typedef typename type::result_t result_t;
            typedef typename type::plain_t  plain_t;
            typedef typename type::choose_t choose_t;
        };
#else
        template <typename T>
        struct to_longest_alternative
        {
            typedef T result_t;
            static result_t const&
            convert(T const& a)  //  Special (end) case
            { return a; }
        };

        template <typename A, typename B>
        struct to_longest_alternative<alternative<A, B> >
        {
            typedef typename to_longest_alternative<A>::result_t    a_t;
            typedef typename to_longest_alternative<B>::result_t    b_t;
            typedef longest_alternative<a_t, b_t>                   result_t;

            static result_t
            convert(alternative<A, B> const& alt) // Recursive case
            {
                return result_t(
                    to_longest_alternative<A>::convert(alt.left()),
                    to_longest_alternative<B>::convert(alt.right()));
            }
        };

        template <typename T>
        struct to_shortest_alternative
        {
            typedef T result_t;
            static result_t const&
            convert(T const& a) //  Special (end) case
            { return a; }
        };

        template <typename A, typename B>
        struct to_shortest_alternative<alternative<A, B> >
        {
            typedef typename to_shortest_alternative<A>::result_t   a_t;
            typedef typename to_shortest_alternative<B>::result_t   b_t;
            typedef shortest_alternative<a_t, b_t>                  result_t;

            static result_t
            convert(alternative<A, B> const& alt) //  Recursive case
            {
                return result_t(
                    to_shortest_alternative<A>::convert(alt.left()),
                    to_shortest_alternative<B>::convert(alt.right()));
            }
        };
#endif
    }
} // namespace spirit

#undef SPIRIT_TYPENAME
#endif

