/*=============================================================================
    Spirit V1.5.0
    Copyright (c) 1998-2002 Spirit-Dev
    http://spirit.sourceforge.net/

    Permission to copy, use, modify, sell and distribute this software
    is granted provided this copyright notice appears in all copies.
    This software is provided "as is" without express or implied
    warranty, and with no claim as to its suitability for any purpose.

    See spirit.hpp for full copyright notices.
=============================================================================*/
#if !defined(SPIRIT_RULE_IPP)
#define SPIRIT_RULE_IPP

#if defined(SPIRIT_DEBUG)
#include <iostream>
#include <string>
//#include "boost/spirit/debug/parser_names.hpp"
#endif

///////////////////////////////////////////////////////////////////////////////
namespace spirit
{
    ///////////////////////////////////////////////////////////////////////////
    namespace impl
    {
        ///////////////////////////////////////////////////////////////////////
        //
        //  abstract_parser class
        //
        ///////////////////////////////////////////////////////////////////////
        template <typename ScannerT, typename RT>
        struct abstract_parser
        {
            abstract_parser() {}
            virtual ~abstract_parser() {}

            virtual RT
            do_parse_virtual(ScannerT const& scan) const = 0;
        };

        ///////////////////////////////////////////////////////////////////////
        //
        //  concrete_parser class
        //
        ///////////////////////////////////////////////////////////////////////
        template <typename ParserT, typename ScannerT, typename RT>
        struct concrete_parser : public abstract_parser<ScannerT, RT>
        {
            concrete_parser(ParserT const& p_) : p(p_) {}
            virtual ~concrete_parser() {}

            virtual RT
            do_parse_virtual(ScannerT const& scan) const
            { return p.parse(scan); }

            typename ParserT::embed_t p;
        };

    } // namespace impl

//////////////////////////////////
template <typename ScannerT, typename ContextT>
inline rule<ScannerT, ContextT>::rule()
: ptr(), id(this) {}

//////////////////////////////////
template <typename ScannerT, typename ContextT>
inline rule<ScannerT, ContextT>::~rule() {}

//////////////////////////////////
template <typename ScannerT, typename ContextT>
inline rule<ScannerT, ContextT>::rule(rule const& other)
: ptr(other.ptr), id(this) {}

//////////////////////////////////
#if defined(SPIRIT_DEBUG)
    template<typename ContextT, typename ScannerT>
    std::string
    parser_name(rule<ContextT, ScannerT> const& p);
#endif

template <typename ScannerT, typename ContextT>
inline rule<ScannerT, ContextT>&
rule<ScannerT, ContextT>::operator=(rule const& other)
{
#if defined(SPIRIT_DEBUG)
    if (other.ptr.get() == 0)
        SPIRIT_DEBUG_OUT << "WARNING: Attempting to alias an undefined rule: "
        << parser_name(*this) << " = " << parser_name(other) << std::endl;
#endif
    ptr = other.ptr; return *this;
}

//////////////////////////////////
template <typename ScannerT, typename ContextT>
inline typename rule<ScannerT, ContextT>::result_t
rule<ScannerT, ContextT>::parse(ScannerT const& scan) const
{
    scanner_t scanwrap(scan);
    context_t ctx(*this);

    ctx.pre_parse(*this, scanwrap);
    result_t hit;

    if (ptr.get())
    {
        typename ScannerT::iterator_t s(scan.first);
        hit = ptr->do_parse_virtual(scan);
        scan.group_match(hit, id, s, scan.first);
    }
    else
        hit = scan.no_match();
    return ctx.post_parse(hit, *this, scanwrap);
}

///////////////////////////////////////////////////////////////////////////////
}   //  namespace Spirit

#endif
