/*=============================================================================
    Spirit V1.5.0
    Copyright (c) 1998-2002 Spirit-Dev
    http://spirit.sourceforge.net/

    Permission to copy, use, modify, sell and distribute this software
    is granted provided this copyright notice appears in all copies.
    This software is provided "as is" without express or implied
    warranty, and with no claim as to its suitability for any purpose.

    See spirit.hpp for full copyright notices.
=============================================================================*/
#if !defined(SPIRIT_PARSER_CONTEXT_HPP)
#define SPIRIT_PARSER_CONTEXT_HPP

///////////////////////////////////////////////////////////////////////////////
namespace spirit
{

struct nil_t;
namespace impl { struct empty_base {}; }

///////////////////////////////////////////////////////////////////////////////
//
//  parser_context class { default context for the rule }
//
///////////////////////////////////////////////////////////////////////////////
struct parser_context
{
    typedef nil_t attr_t;
    typedef impl::empty_base base_t;

    template <typename ParserT>
    parser_context(ParserT const& /*p*/) {}

    template <typename ParserT, typename ScannerT>
    void
    pre_parse(ParserT const& /*p*/, ScannerT const& /*scan*/) {}

    template <typename ResultT, typename ParserT, typename ScannerT>
    ResultT&
    post_parse(ResultT& hit, ParserT const& /*p*/, ScannerT const& /*scan*/)
    { return hit; }
};

///////////////////////////////////////////////////////////////////////////////
//
//  parser_context_helper and parser_scanner_linker classes
//  { helper templates for the rule extendability }
//
//      This classes can be 'overloaded' (defined elsewhere), to plug in
//      additional functionality into the rule parsing process.
//
///////////////////////////////////////////////////////////////////////////////
#if !defined(SPIRIT_PARSER_SCANNER_LINKER_DEFINED)
#define SPIRIT_PARSER_SCANNER_LINKER_DEFINED

template<typename ScannerT>
struct parser_scanner_linker : public ScannerT
{
    parser_scanner_linker(ScannerT const scan_) : ScannerT(scan_) {}
};

#endif // !defined(SPIRIT_PARSER_SCANNER_LINKER_DEFINED)

//////////////////////////////////
#if !defined(SPIRIT_PARSER_CONTEXT_LINKER_DEFINED)
#define SPIRIT_PARSER_CONTEXT_LINKER_DEFINED

template<typename ContextT>
struct parser_context_linker : public ContextT
{
    template <typename ParserT>
    parser_context_linker(ParserT const& p)
    : ContextT(p) {}

    template <typename ParserT, typename ScannerT>
    void pre_parse(ParserT const& p, ScannerT const& scan)
    { ContextT::pre_parse(p, scan); }

    template <typename ResultT, typename ParserT, typename ScannerT>
    ResultT& post_parse(ResultT& hit, ParserT const& p, ScannerT const& scan)
    { return ContextT::post_parse(hit, p, scan); }
};

#endif // !defined(SPIRIT_PARSER_CONTEXT_LINKER_DEFINED)

///////////////////////////////////////////////////////////////////////////////
}   //  namespace Spirit

#endif
