/*=============================================================================
    Spirit V1.5.0
    Copyright (c) 1998-2002 Spirit-Dev
    http://spirit.sourceforge.net/

    Permission to copy, use, modify, sell and distribute this software
    is granted provided this copyright notice appears in all copies.
    This software is provided "as is" without express or implied
    warranty, and with no claim as to its suitability for any purpose.

    See spirit.hpp for full copyright notices.
=============================================================================*/
#if !defined(SPIRIT_RULE_HPP)
#define SPIRIT_RULE_HPP

///////////////////////////////////////////////////////////////////////////////
#include "boost/shared_ptr.hpp"
#include "boost/spirit/core/parser.hpp"
#include "boost/spirit/core/scanner.hpp"
#include "boost/spirit/core/parser_context.hpp"
#include "boost/spirit/core/parser_id.hpp"

///////////////////////////////////////////////////////////////////////////////
namespace spirit
{
//////////////////////////////////
    namespace impl
    {
        template <typename ScannerT, typename RT>
        struct abstract_parser;

        template <typename ParserT, typename ScannerT, typename RT>
        struct concrete_parser;
    }

template <typename ScannerT, typename ContextT>
class rule; // forward declaration

///////////////////////////////////////////////////////////////////////////////
//
//  rule_alias class
//
///////////////////////////////////////////////////////////////////////////////
template <typename ScannerT, typename ContextT>
class rule_alias :
    public parser<rule_alias<ScannerT, ContextT> >
{
public:

    typedef rule_alias<ScannerT, ContextT>              self_t;
    typedef typename rule<ScannerT, ContextT>::result_t result_t;

    template <typename ScannerT2>
    struct result { typedef result_t type; };

    rule_alias(rule<ScannerT, ContextT> const& r_)
    : r(r_) {}

    result_t
    parse(ScannerT const& scan) const
    { return r.parse(scan); }

private:

    rule<ScannerT, ContextT> const& r;
};

///////////////////////////////////////////////////////////////////////////////
//
//  rule class
//
///////////////////////////////////////////////////////////////////////////////
template <typename ScannerT = scanner<>, typename ContextT = parser_context>
class rule :
    public parser<rule<ScannerT, ContextT> >, public ContextT::base_t
{
public:

    typedef rule<ScannerT, ContextT>         self_t;
    typedef rule<ScannerT, ContextT> const&  embed_t;

    typedef parser_scanner_linker<ScannerT> scanner_t;
    typedef parser_context_linker<ContextT> context_t;
    typedef typename context_t::attr_t attr_t;
    typedef typename match_result<ScannerT, attr_t>::type result_t;

    template <typename ScannerT2>
    struct result { typedef result_t type; };

    rule();
    ~rule();
    rule(rule const& other);

    result_t
    parse(ScannerT const& scan) const;

    template <typename ParserT>
    rule(ParserT const& p)
    : ptr(new impl::concrete_parser<ParserT, ScannerT, result_t>(p)) {}

    template <typename ParserT>
    rule& operator=(ParserT const& p)
    {
        ptr.reset(new impl::concrete_parser<ParserT, ScannerT, result_t>(p));
        return *this;
    }

    //  If this is placed above the templatized assignment
    //  operator, VC6 incorrectly complains ambiguity with
    //  r1 = r2, where r1 and r2 are both rules.
    rule& operator=(rule const& other);

    rule_alias<ScannerT, ContextT>
    alias() const
    { return rule_alias<ScannerT, ContextT>(*this); }

    parser_id set_id(parser_id const& id_) 
    { 
        parser_id oldid = id;
        id = id_;
        return oldid;
    }

private:

    typedef impl::abstract_parser<ScannerT, result_t> abstract_parser_t;
    boost::shared_ptr<abstract_parser_t> ptr;
    parser_id id;
};

///////////////////////////////////////////////////////////////////////////////
}   //  namespace Spirit

#include "boost/spirit/core/impl/rule.ipp"
#endif
