/*=============================================================================
    Spirit V1.5.0
    Copyright (c) 1998-2002 Spirit-Dev
    http://spirit.sourceforge.net/

    Permission to copy, use, modify, sell and distribute this software
    is granted provided this copyright notice appears in all copies.
    This software is provided "as is" without express or implied
    warranty, and with no claim as to its suitability for any purpose.

    See spirit.hpp for full copyright notices.
=============================================================================*/
#if !defined(SPIRIT_SUBRULE_HPP)
#define SPIRIT_SUBRULE_HPP

///////////////////////////////////////////////////////////////////////////////
#if defined(_MSC_VER) && !defined(__COMO_VERSION__)
#pragma inline_depth(255)
#pragma inline_recursion(on)
#endif

///////////////////////////////////////////////////////////////////////////////
#include "boost/spirit/core/parser.hpp"
#include "boost/spirit/core/parser_context.hpp"
#include "boost/type_traits.hpp"
#include "boost/spirit/core/impl/subrule.ipp"

#if defined(BOOST_MSVC) && (BOOST_MSVC <= 1300)
#define SPIRIT_TYPENAME
#else
#define SPIRIT_TYPENAME typename
#endif

///////////////////////////////////////////////////////////////////////////////
namespace spirit
{
///////////////////////////////////////////////////////////////////////////////
//
//  subrules_scanner class
//
///////////////////////////////////////////////////////////////////////////////
template <typename ScannerT, typename ListT>
struct subrules_scanner : public ScannerT
{
    typedef ScannerT                            scanner_t;
    typedef ListT                               list_t;
    typedef subrules_scanner<ScannerT, ListT>   self_t;

    subrules_scanner(ScannerT const& scan, ListT const& list_)
    : ScannerT(scan), list(list_) {}

    template <typename PoliciesT>
    struct rebind_policies
    {
        typedef typename rebind_scanner_policies<ScannerT, PoliciesT>::type
            rebind_scanner;
        typedef subrules_scanner<rebind_scanner, ListT> type;
    };

    template <typename PoliciesT>
    subrules_scanner<
        typename rebind_scanner_policies<ScannerT, PoliciesT>::type,
        ListT>
    change_policies(PoliciesT const& policies) const
    {
        typedef subrules_scanner<
            SPIRIT_TYPENAME rebind_scanner_policies<ScannerT, PoliciesT>::type,
            ListT>
        subrules_scanner_t;

        return subrules_scanner_t(
                ScannerT::change_policies(policies),
                list);
    }

    template <typename IteratorT>
    struct rebind_iterator
    {
        typedef typename rebind_scanner_iterator<ScannerT, IteratorT>::type
            rebind_scanner;
        typedef subrules_scanner<rebind_scanner, ListT> type;
    };

    template <typename IteratorT>
    subrules_scanner<
        typename rebind_scanner_iterator<ScannerT, IteratorT>::type,
        ListT>
    change_iterator(IteratorT const& first, IteratorT const &last) const
    {
        typedef subrules_scanner<
            SPIRIT_TYPENAME rebind_scanner_iterator<ScannerT, IteratorT>::type,
            ListT>
        subrules_scanner_t;

        return subrules_scanner_t(
                ScannerT::change_iterator(first, last),
                list);
    }

    ListT const& list;
};

///////////////////////////////////////////////////////////////////////////////
//
//  subrule_scanner type computer class
//
//      This computer ensures that the scanner will not be recursively
//      instantiated if it's not needed.
//
///////////////////////////////////////////////////////////////////////////////
template <typename ScannerT, typename ListT>
struct subrules_scanner_finder
{
	typedef subrules_scanner<ScannerT, ListT> type;
};

#if !defined(BOOST_NO_TEMPLATE_PARTIAL_SPECIALIZATION)
//
//  MSVC doesn't support PTS. The default implementation
//  still works. It just generates a whole lot more
//  template instantiations.

template <typename ScannerT, typename ListT>
struct subrules_scanner_finder<subrules_scanner<ScannerT, ListT>, ListT>
{
	typedef subrules_scanner<ScannerT, ListT> type;
};

#endif

///////////////////////////////////////////////////////////////////////////////
//
//  subrule_list class
//
///////////////////////////////////////////////////////////////////////////////
template <typename FirstT, typename RestT>
struct subrule_list : public parser<subrule_list<FirstT, RestT> >
{
    typedef subrule_list<FirstT, RestT> self_t;
    typedef FirstT                      first_t;
    typedef RestT                       rest_t;

    subrule_list(FirstT const& first_, RestT const& rest_)
    : first(first_), rest(rest_) {}

    template <typename ScannerT>
    struct result
    {
        typedef typename FirstT::def_t parser_t;
        typedef typename parser_result<parser_t, ScannerT>::type type;
    };

    template <typename ScannerT>
    typename parser_result<self_t, ScannerT>::type
    parse(ScannerT const& scan) const
    {
		typedef typename subrules_scanner_finder<ScannerT, self_t>::type
		    subrules_scanner_t;
        subrules_scanner_t g_arg(scan, *this);
        return first.rhs.parse(g_arg);
    }

    template <int ID, typename DefT>
    inline subrule_list<FirstT, subrule_list<subrule_parser<ID, DefT>, RestT> >
    operator,(subrule_parser<ID, DefT> const& rhs)
    {
        return subrule_list<
            FirstT,
            subrule_list<subrule_parser<ID, DefT>, RestT> >
        (first, subrule_list<subrule_parser<ID, DefT>, RestT>(rhs, rest));
    }

    FirstT first;
    RestT rest;
};

///////////////////////////////////////////////////////////////////////////////
//
//  subrule_parser class
//
///////////////////////////////////////////////////////////////////////////////
template <int ID, typename DefT>
struct subrule_parser : public parser<subrule_parser<ID, DefT> >
{
    typedef subrule_parser<ID, DefT>    self_t;
    typedef DefT                        def_t;
    BOOST_STATIC_CONSTANT(int, id = ID);

    subrule_parser(DefT const& rhs_)
    : rhs(rhs_) {}

    template <typename ScannerT>
    struct result
    {
        typedef typename parser_result<DefT, ScannerT>::type type;
    };

    template <typename ScannerT>
    typename parser_result<self_t, ScannerT>::type
    parse(ScannerT const& scan) const
    {
        return rhs.parse(scan);
    }

    template <int ID2, typename DefT2>
    inline subrule_list
    <
        self_t,
        subrule_list<subrule_parser<ID2, DefT2>, nil_t>
    >
    operator,(subrule_parser<ID2, DefT2> const& rhs) const
    {
        return subrule_list
        <
            self_t,
            subrule_list<subrule_parser<ID2, DefT2>, nil_t>
        >
        (*this, subrule_list<subrule_parser<ID2, DefT2>, nil_t>(rhs, nil_t()));
    }

    typename DefT::embed_t rhs;
};

///////////////////////////////////////////////////////////////////////////////
//
//  subrule class
//
///////////////////////////////////////////////////////////////////////////////
template <int ID, typename ContextT = parser_context>
struct subrule : public parser<subrule<ID, ContextT> >, public ContextT::base_t
{
    typedef subrule<ID, ContextT> self_t;
    typedef subrule<ID, ContextT> const&  embed_t;
    
    typedef parser_context_linker<ContextT> context_t;
    typedef typename context_t::attr_t attr_t;

    BOOST_STATIC_CONSTANT(int, id = ID);

    template <typename ScannerT>
    struct result
    {
        typedef 
            typename impl::get_subrule_result<ID, ScannerT, attr_t>::type type;
    };

    template <typename ScannerT>
    typename parser_result<self_t, ScannerT>::type
    parse(ScannerT const& scan) const
    {
        typedef parser_scanner_linker<ScannerT> scanner_t;
        typedef typename parser_result<self_t, ScannerT>::type result_t;

        scanner_t scanwrap(scan);
        context_t ctx(*this);

        ctx.pre_parse(*this, scanwrap);

        result_t result;
        impl::parse_subrule<result_t, ScannerT, ID>::
            do_(result, scan, scan.list);
            
        return ctx.post_parse(result, *this, scanwrap);
    }

    template <typename DefT>
    subrule_parser<ID, DefT>
    operator=(parser<DefT> const& rhs) const
    {
        return subrule_parser<ID, DefT>(rhs.derived());
    }
};

///////////////////////////////////////////////////////////////////////////////

}   //  namespace Spirit

#undef SPIRIT_TYPENAME
#endif

