/*=============================================================================
    Spirit V1.5.0
    Copyright (c) 1998-2002 Spirit-Dev
    http://spirit.sourceforge.net/

    Permission to copy, use, modify, sell and distribute this software
    is granted provided this copyright notice appears in all copies.
    This software is provided "as is" without express or implied
    warranty, and with no claim as to its suitability for any purpose.

    See spirit.hpp for full copyright notices.
=============================================================================*/
#if !defined(SPIRIT_DEBUG_MAIN_HPP)
#define SPIRIT_DEBUG_MAIN_HPP

///////////////////////////////////////////////////////////////////////////
//
//  SPIRIT_ASSERT is used throughout the framework.  It can be overridden
//  by the user.  If SPIRIT_ASSERT_EXCEPTION is thrown, then that will
//  be thrown, otherwise, SPIRIT_ASSERT simply turns into a plain assert()
//
///////////////////////////////////////////////////////////////////////////
#if !defined(SPIRIT_ASSERT)
#if defined(NDEBUG)
    #define SPIRIT_ASSERT(x)
#elif defined (SPIRIT_ASSERT_EXCEPTION)
    #define SPIRIT_ASSERT_AUX(f, l, x) \
    do{ if (!(x)) throw SPIRIT_ASSERT_EXCEPTION(f "(" #l "): " #x); } while(0)
    #define SPIRIT_ASSERT(x) SPIRIT_ASSERT_AUX(__FILE__, __LINE__, x)
#else
    #include <cassert>
    #define SPIRIT_ASSERT(x) assert(x)
#endif
#endif // !defined(SPIRIT_ASSERT)


///////////////////////////////////////////////////////////////////////////
#if defined(SPIRIT_DEBUG)

///////////////////////////////////////////////////////////////////////////////
//
//  Spirit.Debug includes and defines
//
///////////////////////////////////////////////////////////////////////////////

    #include <iostream>

    ///////////////////////////////////////////////////////////////////////////
    //
    //  The SPIRIT_DEBUG_OUT defines the stream object, which should be used
    //  for debug diagnostics. This defaults to std::cout.
    //
    ///////////////////////////////////////////////////////////////////////////
    #if !defined(SPIRIT_DEBUG_OUT)
    #define SPIRIT_DEBUG_OUT std::cout
    #endif

    ///////////////////////////////////////////////////////////////////////////
    //
    //  The SPIRIT_DEBUG_PRINT_SOME constant defines the number of characters
    //  from the stream to be printed for diagnosis. This defaults to the first
    //  20 characters.
    //
    ///////////////////////////////////////////////////////////////////////////
    #if !defined(SPIRIT_DEBUG_PRINT_SOME)
    #define SPIRIT_DEBUG_PRINT_SOME 20
    #endif

    ///////////////////////////////////////////////////////////////////////////
    //
    //  Additional SPIRIT_DEBUG_FLAGS control the level of diagnostics printed
    //  Basic constants are defined in debug/minimal.hpp.
    //
    ///////////////////////////////////////////////////////////////////////////
    #define SPIRIT_DEBUG_FLAGS_NODES        0x0001  // print node diagnostics
    #define SPIRIT_DEBUG_FLAGS_MAX          0xFFFF  // print maximal diagnostics

    #if !defined(SPIRIT_DEBUG_FLAGS)
    #define SPIRIT_DEBUG_FLAGS SPIRIT_DEBUG_FLAGS_MAX
    #endif

    ///////////////////////////////////////////////////////////////////////////
    //
    //  By default all nodes are traced. The following constant may be used to
    //  redefine this default.
    //
    ///////////////////////////////////////////////////////////////////////////
    #if !defined(SPIRIT_DEBUG_TRACENODE)
    #define SPIRIT_DEBUG_TRACENODE          (true)
    #endif // !defined(SPIRIT_DEBUG_TRACENODE)

    ///////////////////////////////////////////////////////////////////////////
    //
    //  Helper macros for giving rules and subrules a name accessible through
    //  parser_name() functions (see parser_names.hpp).
    //
    //  Additionally, the macros SPIRIT_DEBUG_RULE, SPIRIT_DEBUG_NODE and
    //  SPIRIT_DEBUG_GRAMMAR enable/disable the tracing of the corresponding
    //  node accordingly to the PP constant SPIRIT_DEBUG_TRACENODE.
    //
    //  The macros SPIRIT_TRACE_RULE, SPIRIT_TRACE_NODE and
    //  SPIRIT_TRACE_GRAMMAR allow to specify a flag to define, if the
    //  corresponding node is to be traced or not.
    //
    ///////////////////////////////////////////////////////////////////////////
    #if !defined(SPIRIT_DEBUG_RULE)
    #define SPIRIT_DEBUG_RULE(r)    \
        ::spirit::impl::get_node_registry().register_node(&r, #r, SPIRIT_DEBUG_TRACENODE)
    #endif // !defined(SPIRIT_DEBUG_RULE)

    #if !defined(SPIRIT_DEBUG_NODE)
    #define SPIRIT_DEBUG_NODE(r)    \
        ::spirit::impl::get_node_registry().register_node(&r, #r, SPIRIT_DEBUG_TRACENODE)
    #endif // !defined(SPIRIT_DEBUG_NODE)

    #if !defined(SPIRIT_DEBUG_GRAMMAR)
    #define SPIRIT_DEBUG_GRAMMAR(r)    \
        ::spirit::impl::get_node_registry().register_node(&r, #r, SPIRIT_DEBUG_TRACENODE)
    #endif // !defined(SPIRIT_DEBUG_GRAMMAR)

    #if !defined(SPIRIT_TRACE_RULE)
    #define SPIRIT_TRACE_RULE(r, t)    \
        ::spirit::impl::get_node_registry().register_node(&r, #r, (t))
    #endif // !defined(SPIRIT_TRACE_RULE)

    #if !defined(SPIRIT_TRACE_NODE)
    #define SPIRIT_TRACE_NODE(r, t)    \
        ::spirit::impl::get_node_registry().register_node(&r, #r, (t))
    #endif // !defined(SPIRIT_TRACE_NODE)

    #if !defined(SPIRIT_TRACE_GRAMMAR)
    #define SPIRIT_TRACE_GRAMMAR(r, t)    \
        ::spirit::impl::get_node_registry().register_node(&r, #r, (t))
    #endif // !defined(SPIRIT_TRACE_GRAMMAR)

    //////////////////////////////////
    #include "boost/spirit/debug/debug_node.hpp"

#else
    //////////////////////////////////
    #include "boost/spirit/debug/minimal.hpp"

#endif // SPIRIT_DEBUG

#endif

