/*=============================================================================
    Generate XML from a parse tree

    Spirit V1.3

    Copyright (c) 1998-2001 Joel de Guzman
    Copyright (c) 2001 Daniel Nuffer
    Copyright (c) 2001 Hartmut Kaiser

    This software is provided 'as-is', without any express or implied
    warranty. In no event will the copyright holder be held liable for
    any damages arising from the use of this software.

    Permission is granted to anyone to use this software for any purpose,
    including commercial applications, and to alter it and redistribute
    it freely, subject to the following restrictions:

    1.  The origin of this software must not be misrepresented; you must
        not claim that you wrote the original software. If you use this
        software in a product, an acknowledgment in the product documentation
        would be appreciated but is not required.

    2.  Altered source versions must be plainly marked as such, and must
        not be misrepresented as being the original software.

    3.  This notice may not be removed or altered from any source
        distribution.

    URL: http://spirit.sourceforge.net/

=============================================================================*/

#if !defined(PARSE_TREE_XML_HPP)
#define PARSE_TREE_XML_HPP

///////////////////////////////////////////////////////////////////////////////
#include "boost/spirit/spirit_tree_fwd.hpp"

namespace spirit {

///////////////////////////////////////////////////////////////////////////////
// 
//  Dump a parse tree as a xml stream 
//
//      The functions 'tree_to_xml' can be used to output a parse tree as a xml
//      stream into the given ostream. The parameters have the following
//      meaning:
// 
//  mandatory parameters:
//      ostrm       The output stream used for streaming the parse tree.
//      tree        The parse tree to output.
//
//  optional parameters:
//      input_line  The input line from which the parse tree was 
//                  generated (if given, it is used to output a comment 
//                  containing this line).
//      id_to_name  A map, which is used for converting the rule id's contained
//                  in the parse tree to readable strings. Here a auxiliary
//                  associative container can be used, which maps a rule_id to 
//                  a std::string (i.e. a std::map<rule_id, std::string>).
//      get_token_id
//                  A function or functor, which takes an instance of a token 
//                  and which should return a token id (i.e. something like
//                  'int f(char const c)').
//      get_token_value
//                  A function or functor, which takes an instance of a token
//                  and which should return a readable representation of this 
//                  token (i.e. something like 'std::string f(char const c)').
//
//  The structure of the generated xml stream conforms to the DTD given in the
//  file 'parsetree.dtd'. This file is located in the spirit/tree directory. 
//
///////////////////////////////////////////////////////////////////////////////

    template <
        typename TreeNodeT, typename AssocContainerT, 
        typename GetIdT, typename GetValueT
    >
    void tree_to_xml (std::ostream &ostrm, TreeNodeT const &tree, 
        std::string const &input_line, AssocContainerT const& id_to_name, 
        GetIdT const &get_token_id, GetValueT const &get_token_value);

    template <typename TreeNodeT, typename AssocContainerT>
    void tree_to_xml (std::ostream &ostrm, TreeNodeT const &tree, 
        std::string const &input_line, AssocContainerT const& id_to_name);

    template <typename TreeNodeT>
    void tree_to_xml (std::ostream &ostrm, TreeNodeT const &tree,
        std::string const &input_line = "");

} // end of namespace spirit

#include "boost/spirit/tree/impl/parse_tree_xml.ipp"

#endif // !defined(PARSE_TREE_XML_HPP)
