/*=============================================================================
    Spirit V1.5.0
    Copyright (c) 1998-2002 Spirit-Dev
    http://spirit.sourceforge.net/

    Permission to copy, use, modify, sell and distribute this software
    is granted provided this copyright notice appears in all copies.
    This software is provided "as is" without express or implied
    warranty, and with no claim as to its suitability for any purpose.

    See spirit.hpp for full copyright notices.
=============================================================================*/
#ifndef SPIRIT_REGEX_IPP
#define SPIRIT_REGEX_IPP

///////////////////////////////////////////////////////////////////////////////
#include "boost/spirit/core/impl/primitives.ipp"

///////////////////////////////////////////////////////////////////////////////
namespace spirit {

namespace impl {

///////////////////////////////////////////////////////////////////////////////
//
//  rx_parser class
//
///////////////////////////////////////////////////////////////////////////////
template <typename IteratorT = char const*>
class rx_parser : public parser<rx_parser<IteratorT> >
{
public:
    typedef typename std::iterator_traits<IteratorT>::value_type value_type;
    typedef std::basic_string<value_type> string_t;
    typedef rx_parser<IteratorT> self_t;

    rx_parser(IteratorT first, IteratorT last)
    { rxstr = string_t("^") + string_t(first, last); }

    rx_parser(IteratorT first)
    { rxstr = string_t("^") + string_t(first, impl::get_last(first)); }

    template <typename ScannerT>
    typename parser_result<self_t, ScannerT>::type
    parse(ScannerT const& scan) const
    {
        boost::match_results<IteratorT> what;
        boost::regex_search(scan.first, scan.last, what, rxstr, 
            boost::match_default);
        
        bool matched = what[0].matched;
        unsigned count = what[0].length();
        if (0 == count)
            return scan.no_match();
            
        scan.first = what[0].second;
        return scan.create_match(count, nil_t(), what[0].first, scan.first);
    }

private:
    boost::reg_expression<value_type> rxstr;    // regular expression to match
};

}   // namespace impl

///////////////////////////////////////////////////////////////////////////////
}   // namespace spirit

#endif // SPIRIT_REGEX_IPP
