/*=============================================================================
    Spirit V1.5.0
    Copyright (c) 1998-2002 Spirit-Dev
    http://spirit.sourceforge.net/

    Permission to copy, use, modify, sell and distribute this software
    is granted provided this copyright notice appears in all copies.
    This software is provided "as is" without express or implied
    warranty, and with no claim as to its suitability for any purpose.

    See spirit.hpp for full copyright notices.
=============================================================================*/
#ifndef SPIRIT_REGEX_HPP
#define SPIRIT_REGEX_HPP

#include "boost/spirit/core/impl/msvc.hpp"

#if defined(BOOST_MSVC) && (BOOST_MSVC <= 1300)
#define SPIRIT_IT_NS impl
#else
#define SPIRIT_IT_NS std
#endif

///////////////////////////////////////////////////////////////////////////////
//
//  Include the regular expression library of boost (Boost.Regex)
//
//  Note though, that this library is not distributed with Spirit. You have to
//  obtain a separate copy from http://www.boost.org.
//
///////////////////////////////////////////////////////////////////////////////
#if defined(SPIRIT_NO_REGEX_LIB)
//
//  Include all the Boost.regex library. Please note that this will not work,
//  if you are using the boost/spirit/regex.hpp header from more than one
//  translation units.
//
#define BOOST_REGEX_NO_LIB
#define BOOST_REGEX_STATIC_LINK
#include <boost/regex.hpp>
#include <boost/regex/src.cpp>

#else
//
//  Include the Boost.Regex headers only. Note, that you will have to link your
//  application against the Boost.Regex library as described in the related
//  documentation.
//
#include <boost/regex.hpp>
#endif // defined(SPIRIT_NO_REGEX_LIB)

#include <boost/static_assert.hpp>

///////////////////////////////////////////////////////////////////////////////
#include "boost/spirit/core/impl/parser_type.hpp"
#include "boost/spirit/core/parser.hpp"
#include "boost/spirit/utility/impl/regex.ipp"

namespace spirit {

///////////////////////////////////////////////////////////////////////////////
// rxstrlit class
template <typename IteratorT = char const *>
struct rxstrlit :
    public parser<rxstrlit<IteratorT> > {

    typedef rxstrlit<IteratorT> self_t;

    rxstrlit(IteratorT first, IteratorT last)
    : rx(first, last) {}
    rxstrlit(IteratorT first)
    : rx(first) {}

    template <typename ScannerT>
    typename parser_result<self_t, ScannerT>::type
    parse(ScannerT const& scan) const
    {
    //  Due to limitations in the boost::regex library the iterators wrapped in
    //  the ScannerT object should be at least bidirectional iterators. Plain
    //  forward iterators do not work here.
        typedef typename ScannerT::iterator_t iterator_t;
        typedef
            typename SPIRIT_IT_NS::iterator_traits<iterator_t>::iterator_category
            iterator_category;

#if !defined(__MWERKS__) && (__MWERKS__ > 0x2407)
// CW can't use boost::is_base_and_derived

        BOOST_STATIC_ASSERT((
            boost::is_same<std::bidirectional_iterator_tag,
                iterator_category>::value ||
            boost::is_base_and_derived<std::bidirectional_iterator_tag,
                iterator_category>::value
        ));
#endif

        typedef typename parser_result<self_t, ScannerT>::type result_t;
        return impl::contiguous_parser_parse<result_t>(rx, scan, scan);
    }

private:
    impl::rx_parser<IteratorT> rx;   // boost regular expression parser
};

///////////////////////////////////////////////////////////////////////////////
// Generator functions
template <typename CharT>
inline rxstrlit<CharT const*>
regex_p(CharT const* str)
{ return rxstrlit<CharT const*>(str); }

//////////////////////////////////
template <typename IteratorT>
inline rxstrlit<IteratorT>
regex_p(IteratorT first, IteratorT last)
{ return rxstrlit<IteratorT>(first, last); }

}   //  namespace Spirit

#undef SPIRIT_IT_NS

#endif // SPIRIT_REGEX_HPP
